
//+------------------------------------------------------------------+
//|                                            GoldenRatioEA.mq5     |
//|                          نوشته شده برای MetaTrader 5            |
//|   اکسپرت با سیگنال خرید/فروش و تنظیم حد سود و ضرر با نسبت طلایی |
//+------------------------------------------------------------------+
#property copyright "Code Copilot"
#property link      "https://1lm.me/cc"
#property version   "1.00"

// پارامترهای اکسپرت
input int Periods[] = {8, 13, 21, 34, 55};  // دوره‌ها
input double InitialLotSize = 0.1;          // حجم اولیه معاملات
input double LotIncreaseRate = 0.1;         // نرخ افزایش (10%)
input int IncreaseAfterTrades = 5;          // افزایش حجم پس از چند معامله
input double Slippage = 3;                  // اسلیپیج
input double StopLossMultiplier = 0.618;    // ضریب حد ضرر
input double TakeProfitMultiplier = 1.618;  // ضریب حد سود

// شمارنده معاملات
int TotalTrades = 0; // تعداد کل معاملات
double CurrentLotSize = 0.0;

//+------------------------------------------------------------------+
//| تابع تنظیم حجم معاملات                                           |
//+------------------------------------------------------------------+
void UpdateLotSize()
  {
   // اگر تعداد معاملات به شرط افزایش رسید
   if(TotalTrades > 0 && TotalTrades % IncreaseAfterTrades == 0)
     {
      CurrentLotSize *= (1.0 + LotIncreaseRate); // افزایش حجم به میزان LotIncreaseRate
      PrintFormat("افزایش حجم معاملات به %.2f لات پس از %d معامله", CurrentLotSize, TotalTrades);
     }
  }

//+------------------------------------------------------------------+
//| تابع محاسبه سطوح فیبوناچی                                        |
//+------------------------------------------------------------------+
void CalculateFibonacciLevels(int period, double &golden, double &inverse_golden, double &extension, double &stop_loss, double &take_profit)
  {
   double max_price = iHigh(NULL, 0, period);
   double min_price = iLow(NULL, 0, period);
   double range = max_price - min_price;

   golden = max_price - range * 0.618;                    // سطح اصلاحی 0.618
   inverse_golden = min_price + range * 1.618;            // سطح گسترشی 1.618
   extension = (max_price + min_price) / 2 + range * 1.618; // گسترش اضافی
   stop_loss = range * StopLossMultiplier;                // حد ضرر
   take_profit = range * TakeProfitMultiplier;            // حد سود
  }

//+------------------------------------------------------------------+
//| تابع بررسی سیگنال‌های خرید و فروش                                |
//+------------------------------------------------------------------+
void CheckForSignals()
  {
   for(int i = 0; i < ArraySize(Periods); i++)
     {
      int period = Periods[i];
      if(Bars < period) continue;

      double golden, inverse_golden, extension, stop_loss, take_profit;
      CalculateFibonacciLevels(period, golden, inverse_golden, extension, stop_loss, take_profit);

      // بررسی سیگنال خرید
      if(Close[1] <= inverse_golden && Close[0] > inverse_golden)
        {
         if(PositionSelect("BUY")) continue; // اگر پوزیشن خرید باز است، سیگنال جدید ندهد
         OpenBuy(period, stop_loss, take_profit);
        }

      // بررسی سیگنال فروش
      if(Close[1] >= golden && Close[0] < golden)
        {
         if(PositionSelect("SELL")) continue; // اگر پوزیشن فروش باز است، سیگنال جدید ندهد
         OpenSell(period, stop_loss, take_profit);
        }
     }
  }

//+------------------------------------------------------------------+
//| تابع باز کردن معامله خرید                                        |
//+------------------------------------------------------------------+
void OpenBuy(int period, double stop_loss, double take_profit)
  {
   double price = SymbolInfoDouble(_Symbol, SYMBOL_ASK);
   double sl = price - stop_loss;  // حد ضرر
   double tp = price + take_profit; // حد سود

   if(OrderSend(_Symbol, OP_BUY, CurrentLotSize, price, Slippage, sl, tp, "GoldenRatioEA Buy", 0, 0, clrLime) > 0)
     {
      TotalTrades++; // افزایش شمارنده معاملات
      UpdateLotSize(); // به‌روزرسانی حجم معاملات
     }
   else
      Print("خطا در باز کردن پوزیشن خرید: ", GetLastError());
  }

//+------------------------------------------------------------------+
//| تابع باز کردن معامله فروش                                        |
//+------------------------------------------------------------------+
void OpenSell(int period, double stop_loss, double take_profit)
  {
   double price = SymbolInfoDouble(_Symbol, SYMBOL_BID);
   double sl = price + stop_loss;  // حد ضرر
   double tp = price - take_profit; // حد سود

   if(OrderSend(_Symbol, OP_SELL, CurrentLotSize, price, Slippage, sl, tp, "GoldenRatioEA Sell", 0, 0, clrRed) > 0)
     {
      TotalTrades++; // افزایش شمارنده معاملات
      UpdateLotSize(); // به‌روزرسانی حجم معاملات
     }
   else
      Print("خطا در باز کردن پوزیشن فروش: ", GetLastError());
  }

//+------------------------------------------------------------------+
//| تابع مدیریت معاملات باز                                          |
//+------------------------------------------------------------------+
void ManageOpenPositions()
  {
   for(int i = PositionsTotal() - 1; i >= 0; i--)
     {
      ulong ticket = PositionGetTicket(i);
      string symbol = PositionGetString(POSITION_SYMBOL);
      if(symbol != _Symbol) continue;

      double stop_loss = PositionGetDouble(POSITION_SL);
      double take_profit = PositionGetDouble(POSITION_TP);
      double current_price = SymbolInfoDouble(_Symbol, SYMBOL_BID);

      // بررسی حد سود و ضرر
      if((current_price >= take_profit) || (current_price <= stop_loss))
         PositionClose(ticket);
     }
  }

//+------------------------------------------------------------------+
//| تابع اصلی اکسپرت                                                 |
//+------------------------------------------------------------------+
int OnInit()
  {
   CurrentLotSize = InitialLotSize; // تنظیم حجم اولیه معاملات
   return(INIT_SUCCEEDED);
  }

int OnTick()
  {
   CheckForSignals();       // بررسی سیگنال‌های جدید
   ManageOpenPositions();   // مدیریت معاملات باز
   return 0;
  }
//+------------------------------------------------------------------+
